//   Copyright (c)  2007  John Abbott and Anna Bigatti

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.

#include "CoCoA/library.H"

// #include <iostream>
using std::endl;
using std::clog;
#include <sstream>
using std::stringstream;
// #include <memory>
using std::auto_ptr;
// #include <string>
using std::string;
// #include <vector>
using std::vector;

namespace CoCoA
{

  // sublibrary of CoCoALib for groebner related operations
  // by M.Caboara
  const ServerOpBase::LibraryInfo& CoCoALib_groebner()
  {
    static ServerOpBase::LibraryInfo UniqueValue("CoCoALib", BuildInfo::version, "groebner");
    return UniqueValue;
  }
  
  // sublibrary of CoCoALib for approx point
  // by J.Abbott, ML.Torrente
  const ServerOpBase::LibraryInfo& CoCoALib_approx()
  {
    static ServerOpBase::LibraryInfo UniqueValue("CoCoALib", BuildInfo::version, "approx");
    return UniqueValue;
  }
  
  // sublibrary of CoCoALib for monomial (squarefree) ideals
  // by M.Caboara, E.Saenz-de-Cabezon
  const ServerOpBase::LibraryInfo& CoCoALib_combinatorics()
  {
    static ServerOpBase::LibraryInfo UniqueValue("CoCoALib", BuildInfo::version, "combinatorics");
    return UniqueValue;
  }
  

  // ---- CoCoA/GOperations.H  by  M.Caboara ----
  class ModuleGBasis: public ServerOpBase
  {
  public:
    ModuleGBasis(): ServerOpBase(CoCoALib_groebner()) {};
    ~ModuleGBasis() {};
    void myOutputSelf(std::ostream& out) const {out << "ModuleGBasis";}
    void myReadArgs(std::istream& in);
    void myCompute()  { ComputeGBasis(myResVL, myInVL); }
    void myWriteResult(std::ostream& out) const { WriteVectorListInVar(out, ourVarName4, myResVL); }
    void myClear() { myInVL.clear(); myResVL.clear(); }
  private:
    VectorList myInVL, myResVL;
  };


  void ModuleGBasis::myReadArgs(std::istream& in)
  {
    const FreeModule FM(ReadFreeModule(in, GetTag));
    ReadVectorList(in, myInVL, FM, GetTag);
  }


  // ---- CoCoA/GOperations.H  by  M.Caboara ----
  class ModuleLT: public ServerOpBase
  {
  public:
    ModuleLT(): ServerOpBase(CoCoALib_groebner()) {};
    ~ModuleLT() {};
    void myOutputSelf(std::ostream& out) const {out << "ModuleLT";}
    void myReadArgs(std::istream& in);
    void myCompute() { ComputeLT(myResVL, myInVL); }
    void myWriteResult(std::ostream& out) const { WriteVectorListInVar(out, ourVarName4, myResVL); }
    void myClear() { myInVL.clear(); myResVL.clear(); }
  private:
    VectorList myInVL, myResVL;
  };


  void ModuleLT::myReadArgs(std::istream& in)
  {
    const FreeModule FM(ReadFreeModule(in, GetTag));
    ReadVectorList(in, myInVL, FM, GetTag);
  }

  // ---- CoCoA/GOperations.H  by  M.Caboara ----
  class ModuleSyzygy: public ServerOpBase
  {
  public:
    ModuleSyzygy(): ServerOpBase(CoCoALib_groebner()) {};
    ~ModuleSyzygy() {};
    void myOutputSelf(std::ostream& out) const {out << "ModuleSyzygy";}
    void myReadArgs(std::istream& in);
    void myCompute() { ComputeSyz(myResVL, myInVL); }
    void myWriteResult(std::ostream& out) const { WriteVectorListInVar(out, ourVarName4, myResVL); }
    void myClear() { myInVL.clear(); myResVL.clear(); }
  private:
    VectorList myInVL, myResVL;
  };


  void ModuleSyzygy::myReadArgs(std::istream& in)
  {
    const FreeModule FM(ReadFreeModule(in, GetTag));
    ReadVectorList(in, myInVL, FM, GetTag);
  }

  // ---- CoCoA/GOperations.H  by  M.Caboara ----
  class ModuleIntersection: public ServerOpBase
  {
  public:
    ModuleIntersection(): ServerOpBase(CoCoALib_groebner()) {};
    ~ModuleIntersection() {};
    void myOutputSelf(std::ostream& out) const {out << "ModuleIntersection";}
    void myReadArgs(std::istream& in);
    void myCompute() { ComputeIntersection(myResVL, myInVL1, myInVL2); }
    void myWriteResult(std::ostream& out) const { WriteVectorListInVar(out, ourVarName4, myResVL); }
    void myClear() { myInVL1.clear(); myInVL2.clear(); myResVL.clear(); }
  private:
    VectorList myInVL1, myInVL2, myResVL;
  };


  void ModuleIntersection::myReadArgs(std::istream& in)
  {
    const FreeModule FM(ReadFreeModule(in, GetTag));
    ReadVectorList(in, myInVL1, FM, GetTag);
    ReadVectorList(in, myInVL2, FM, GetTag);
  }

  // ---- CoCoA/GOperations.H  by  M.Caboara ----
  class ColonModMod: public ServerOpBase
  {
  public:
    ColonModMod(): ServerOpBase(CoCoALib_groebner()) {};
    ~ColonModMod() {};
    void myOutputSelf(std::ostream& out) const {out << "ColonModMod";}
    void myReadArgs(std::istream& in);
    void myCompute() { ComputeCColon(myResPL, myInVL1, myInVL2); }
    void myWriteResult(std::ostream& out) const { WritePolyListInVar(out, ourVarName4, myResPL); }
    void myClear() { myInVL1.clear(); myInVL2.clear(); myResPL.clear(); }
  private:
    VectorList myInVL1, myInVL2;
    PolyList myResPL;
  };


  void ColonModMod::myReadArgs(std::istream& in)
  {
    const FreeModule FM(ReadFreeModule(in, GetTag));
    ReadVectorList(in, myInVL1, FM, GetTag);
    ReadVectorList(in, myInVL2, FM, GetTag);
  }

  // ---- CoCoA/GOperations.H  by  M.Caboara ----
  class ModuleSaturation: public ServerOpBase
  {
  public:
    ModuleSaturation(): ServerOpBase(CoCoALib_groebner()) {};
    ~ModuleSaturation() {};
    void myOutputSelf(std::ostream& out) const {out << "ModuleSaturation";}
    void myReadArgs(std::istream& in);
    void myCompute() { ComputeSSaturation(myResVL, myInVL, myInPL); }
    void myWriteResult(std::ostream& out) const { WriteVectorListInVar(out, ourVarName4, myResVL); }
    void myClear() { myInVL.clear(); myInPL.clear(); myResVL.clear(); }
  private:
    VectorList myInVL, myResVL;
    PolyList myInPL;
  };


  void ModuleSaturation::myReadArgs(std::istream& in)
  {
    const FreeModule FM(ReadFreeModule(in, GetTag));
    const SparsePolyRing P(AsSparsePolyRing(BaseRing(FM)));
    ReadVectorList(in, myInVL, FM, GetTag);
    ReadPolyList(in, myInPL, P, GetTag);
  }

  // ---- CoCoA/GOperations.H  by  M.Caboara ----
  class IdealSaturation: public ServerOpBase
  {
  public:
    IdealSaturation(): ServerOpBase(CoCoALib_groebner()) {};
    ~IdealSaturation() {};
    void myOutputSelf(std::ostream& out) const {out << "IdealSaturation";}
    void myReadArgs(std::istream& in);
    void myCompute() { ComputeSSaturation(myResPL, myInPL1, myInPL2); }
    void myWriteResult(std::ostream& out) const { WritePolyListInVar(out, ourVarName4, myResPL); }
    void myClear() { myInPL1.clear(); myInPL2.clear(); myResPL.clear(); }
  private:
    PolyList myInPL1, myInPL2, myResPL;
  };


  void IdealSaturation::myReadArgs(std::istream& in)
  {
    const SparsePolyRing P(ReadPolyRing(in, GetTag));
    ReadPolyList(in, myInPL1, P, GetTag);
    ReadPolyList(in, myInPL2, P, GetTag);
  }

  // ---- CoCoA/GOperations.H  by  M.Caboara ----
  // class ColonModId: public ServerOpBase
  // {
  // public:
  //   ColonModId(): ServerOpBase(CoCoALib_groebner()) {};
  //   ~ColonModId() {};
  //   void myOutputSelf(std::ostream& out) const {out << "ColonModId";}
  //   void myReadArgs(std::istream& in);
  //   void myCompute() { ComputeCColon(myResVL, myInVL, myInPL); }
  //   void myWriteResult(std::ostream& out) const { WriteVectorListInVar(out, ourVarName4, myResVL); }
  //  void myClear() { myInVL.clear(); myInPL.clear(); myResVL.clear(); }
  // private:
  //   VectorList myInVL, myResVL;
  //   PolyList myInPL;
  // };


  // void ColonModId::myReadArgs(std::istream& in)
  // {
  //   const FreeModule FM(ReadFreeModule(in, GetTag));
  //   const SparsePolyRing P(AsSparsePolyRing(BaseRing(FM)));
  //   ReadVectorList(myInVL, FM, GetTag);
  //   ReadPolyList(myInPL, P, GetTag);
  // }

  // ---- CoCoA/GOperations.H  by  M.Caboara ----
  class ColonIdId: public ServerOpBase
  {
  public:
    ColonIdId(): ServerOpBase(CoCoALib_groebner()) {};
    ~ColonIdId() {};
    void myOutputSelf(std::ostream& out) const {out << "ColonIdId";}
    void myReadArgs(std::istream& in);
    void myCompute() { ComputeCColon(myResPL, myInPL1, myInPL2); }
    void myWriteResult(std::ostream& out) const { WritePolyListInVar(out, ourVarName4, myResPL); }
    void myClear() { myInPL1.clear(); myInPL2.clear(); myResPL.clear(); }
  private:
    PolyList myInPL1, myInPL2, myResPL;
  };


  void ColonIdId::myReadArgs(std::istream& in)
  {
    const SparsePolyRing P(ReadPolyRing(in, GetTag));
    ReadPolyList(in, myInPL1, P, GetTag);
    ReadPolyList(in, myInPL2, P, GetTag);
  }

  // ---- CoCoA/GOperations.H  by  M.Caboara ----
  class ModuleElim: public ServerOpBase
  {
  public:
    ModuleElim(): ServerOpBase(CoCoALib_groebner()) {};
    ~ModuleElim() {};
    void myOutputSelf(std::ostream& out) const {out << "ModuleElim";}
    void myReadArgs(std::istream& in);
    void myCompute() { ComputeElim(myResVL, myInVL, *myInElimPPPtr); }
    void myWriteResult(std::ostream& out) const { WriteVectorListInVar(out, ourVarName4, myResVL); }
    void myClear() { myInVL.clear(); myInElimPPPtr.reset(0); myResVL.clear(); }
  private:
    VectorList myInVL, myResVL;
    auto_ptr<PPMonoidElem> myInElimPPPtr;
  };


  void ModuleElim::myReadArgs(std::istream& in)
  {
    const FreeModule FM(ReadFreeModule(in, GetTag));
    const SparsePolyRing P(AsSparsePolyRing(BaseRing(FM)));
    ReadVectorList(in, myInVL, FM, GetTag);
    PolyList PL;
    myInElimPPPtr.reset(new PPMonoidElem(PPM(P)));
    ReadPolyList(in, PL, P, GetTag);
    for (PolyList::const_iterator it=PL.begin(); it != PL.end(); ++it)
      (*myInElimPPPtr) *= LPP(*it);
  }

  // ---- CoCoA/GOperations.H  by  M.Caboara ----
  class ModuleHomog: public ServerOpBase
  {
  public:
    ModuleHomog(): ServerOpBase(CoCoALib_groebner()) {};
    ~ModuleHomog() {};
    void myOutputSelf(std::ostream& out) const {out << "ModuleHomog";}
    void myReadArgs(std::istream& in);
    void myCompute() { ComputeHomogenization(myResVL, myInVL, myInHomIndets); }
    void myWriteResult(std::ostream& out) const { WriteVectorListInVar(out, ourVarName4, myResVL); }
    void myClear() { myInVL.clear(); myInHomIndets.clear(); myResVL.clear(); }
  private:
    VectorList myInVL, myResVL;
    PolyList myInHomIndets;
  };


  void ModuleHomog::myReadArgs(std::istream& in)
  {
    const FreeModule FM(ReadFreeModule(in, GetTag));
    const SparsePolyRing P(AsSparsePolyRing(BaseRing(FM)));
    ReadVectorList(in, myInVL, FM, GetTag);
    ReadPolyList(in, myInHomIndets, P, GetTag);
  }

  // ---- CoCoA/GOperations.H  by  M.Caboara ----
  class IdealGBasis: public ServerOpBase
  {
  public:
    IdealGBasis(): ServerOpBase(CoCoALib_groebner()) {};
    ~IdealGBasis() {};
    void myOutputSelf(std::ostream& out) const {out << "IdealGBasis";}
    void myReadArgs(std::istream& in);
    void myCompute() { ComputeGBasis(myResPL, myInPL); }
    void myWriteResult(std::ostream& out) const { WritePolyListInVar(out, ourVarName4, myResPL); }
    void myClear() { myInPL.clear(); myResPL.clear(); }
  private:
    PolyList myInPL, myResPL;
  };


  void IdealGBasis::myReadArgs(std::istream& in)
  {
    const SparsePolyRing P(ReadPolyRing(in, GetTag));
    ReadPolyList(in, myInPL, P, GetTag);
  }

 // ---- CoCoA/GOperations.H  by  M.Caboara ----
  class IdealSATGBasis: public ServerOpBase
  {
  public:
    IdealSATGBasis(): ServerOpBase(CoCoALib_groebner()) {};
    ~IdealSATGBasis() {};
    void myOutputSelf(std::ostream& out) const {out << "IdealSATGBasis";}
    void myReadArgs(std::istream& in);
    void myCompute() { ComputeGBasis(myResPL, myInPL); }
    void myWriteResult(std::ostream& out) const { WritePolyListInVar(out, ourVarName4, myResPL); }
    void myClear() { myInPL.clear(); myResPL.clear(); }
  private:
    PolyList myInPL, myResPL;
  };

  void IdealSATGBasis::myReadArgs(std::istream& in)
  {
    const SparsePolyRing P(ReadPolyRing(in, GetTag));
    ReadPolyList(in, myInPL, P, GetTag);
  }
  
// ---- CoCoA/GOperations.H  by  M.Caboara ----
  class IdealSATMixGBasis: public ServerOpBase
  {
  public:
    IdealSATMixGBasis(): ServerOpBase(CoCoALib_groebner()) {};
    ~IdealSATMixGBasis() {};
    void myOutputSelf(std::ostream& out) const {out << "IdealSATMixGBasis";}
    void myReadArgs(std::istream& in);
    void myCompute() { ComputeGBasis(myResPL, myInPL); }
    void myWriteResult(std::ostream& out) const { WritePolyListInVar(out, ourVarName4, myResPL); }
    void myClear() { myInPL.clear(); myResPL.clear(); }
  private:
    PolyList myInPL, myResPL;
  };

  void IdealSATMixGBasis::myReadArgs(std::istream& in)
  {
    const SparsePolyRing P(ReadPolyRing(in, GetTag));
    ReadPolyList(in, myInPL, P, GetTag);
  }
  
  // ---- CoCoA/GOperations.H  by  M.Caboara ----
  class IdealLT: public ServerOpBase
  {
  public:
    IdealLT(): ServerOpBase(CoCoALib_groebner()) {};
    ~IdealLT() {};
    void myOutputSelf(std::ostream& out) const {out << "IdealLT";}
    void myReadArgs(std::istream& in);
    void myCompute() { ComputeLT(myResPL, myInPL); }
    void myWriteResult(std::ostream& out) const { WritePolyListInVar(out, ourVarName4, myResPL); }
    void myClear() { myInPL.clear(); myResPL.clear(); }
  private:
    PolyList myInPL, myResPL;
  };


  void IdealLT::myReadArgs(std::istream& in)
  {
    const SparsePolyRing P(ReadPolyRing(in, GetTag));
    ReadPolyList(in, myInPL, P, GetTag);
  }

  // ---- CoCoA/GOperations.H  by  M.Caboara ----
  class IdealSyzygy: public ServerOpBase
  {
  public:
    IdealSyzygy(): ServerOpBase(CoCoALib_groebner()) {};
    ~IdealSyzygy() {};
    void myOutputSelf(std::ostream& out) const {out << "IdealSyzygy";}
    void myReadArgs(std::istream& in);
    void myCompute() { ComputeSyz(myResVL, myInPL); }
    void myWriteResult(std::ostream& out) const { WriteVectorListInVar(out, ourVarName4, myResVL); }
    void myClear() { myInPL.clear(); myResVL.clear(); }
  private:
    PolyList myInPL;
    VectorList myResVL;
  };


  void IdealSyzygy::myReadArgs(std::istream& in)
  {
    const SparsePolyRing P(ReadPolyRing(in, GetTag));
    ReadPolyList(in, myInPL, P, GetTag);
  }

  // ---- CoCoA/GOperations.H  by  M.Caboara ----
  class IdealIntersection: public ServerOpBase
  {
  public:
    IdealIntersection(): ServerOpBase(CoCoALib_groebner()) {};
    ~IdealIntersection() {};
    void myOutputSelf(std::ostream& out) const {out << "IdealIntersection";}
    void myReadArgs(std::istream& in);
    void myCompute() { ComputeIntersection(myResPL, myInPL1, myInPL2); }
    void myWriteResult(std::ostream& out) const { WritePolyListInVar(out, ourVarName4, myResPL); }
    void myClear() { myInPL1.clear(); myInPL2.clear(); myResPL.clear(); }
  private:
    PolyList myInPL1, myInPL2, myResPL;
  };


  void IdealIntersection::myReadArgs(std::istream& in)
  {
    const SparsePolyRing P(ReadPolyRing(in, GetTag));
    ReadPolyList(in, myInPL1, P, GetTag);
    ReadPolyList(in, myInPL2, P, GetTag);
  }

  // ---- CoCoA/GOperations.H  by  M.Caboara ----
  class IdealElim: public ServerOpBase
  {
  public:
    IdealElim(): ServerOpBase(CoCoALib_groebner()) {};
    ~IdealElim() {};
    void myOutputSelf(std::ostream& out) const {out << "IdealElim";}
    void myReadArgs(std::istream& in);
    void myCompute() { ComputeElim(myResPL, myInPL, *myInElimPPPtr); }
    void myWriteResult(std::ostream& out) const { WritePolyListInVar(out, ourVarName4, myResPL); }
    void myClear() { myInPL.clear(); myResPL.clear(); }
  private:
    PolyList myInPL, myResPL;
    auto_ptr<PPMonoidElem> myInElimPPPtr;
  };


  void IdealElim::myReadArgs(std::istream& in)
  {
    const SparsePolyRing P(ReadPolyRing(in, GetTag));
    ReadPolyList(in, myInPL, P, GetTag);
    PolyList PL;
    PPMonoidElem t(PPM(P));
    ReadPolyList(in, PL, P, GetTag);
    for (PolyList::const_iterator it=PL.begin(); it != PL.end(); ++it)
    {
      if ( !IsIndet(*it) )
        CoCoA_ERROR("Expected indet", "IdealElim::myReadArgs");
      t *= LPP(*it);
    }
    myInElimPPPtr.reset(new PPMonoidElem(t));
  }

  // ---- CoCoA/GOperations.H  by  M.Caboara ----
  class IdealHomog: public ServerOpBase
  {
  public:
    IdealHomog(): ServerOpBase(CoCoALib_groebner()) {};
    ~IdealHomog() {};
    void myOutputSelf(std::ostream& out) const {out << "IdealHomog";}
    void myReadArgs(std::istream& in);
    void myCompute() { ComputeHomogenization(myResPL, myInPL, myInHomIndets); }
    void myWriteResult(std::ostream& out) const { WritePolyListInVar(out, ourVarName4, myResPL); }
    void myClear() { myInPL.clear(); myInHomIndets.clear(); myResPL.clear(); }
  private:
    PolyList myInPL, myInHomIndets, myResPL;
  };


  void IdealHomog::myReadArgs(std::istream& in)
  {
    const SparsePolyRing P(ReadPolyRing(in, GetTag));
    ReadPolyList(in, myInPL, P, GetTag);
    ReadPolyList(in, myInHomIndets, P, GetTag);
  }

  // ---- CoCoA/TmpF5.H  by  A.Arri ----
  class F5GBasis: public ServerOpBase
  {
  public:
    F5GBasis(): ServerOpBase(CoCoALib_groebner()) {};
    ~F5GBasis() {};
    void myOutputSelf(std::ostream& out) const {out << "F5GBasis";}
    void myReadArgs(std::istream& in);
    void myCompute() { F5(myResPL, myInPL); }
    void myWriteResult(std::ostream& out) const { WritePolyListInVar(out, ourVarName4, myResPL); }
    void myClear() { myInPL.clear(); myResPL.clear(); }
  private:
    PolyList myInPL, myResPL;
  };


  void F5GBasis::myReadArgs(std::istream& in)
  {
    const SparsePolyRing P(ReadPolyRing(in, GetTag));
    ReadPolyList(in, myInPL, P, GetTag);
  }

  // ---- CoCoA/TmpIsTree.H  by  M.Caboara ----
  class IsTreeNoOpt: public ServerOpBase
  {
  public:
    IsTreeNoOpt(): ServerOpBase(CoCoALib_combinatorics()) {};
    ~IsTreeNoOpt() {};
    void myOutputSelf(std::ostream& out) const {out << "IsTreeNoOpt";}
    void myReadArgs(std::istream& in);
    void myCompute() { myOutCycle = myInComplex.myIsTreeNoOpt(); }
    void myWriteResult(std::ostream& out) const { WritePolyListInVar(out, ourVarName4, FacetList2PolyList(*myRingPtr,myOutCycle)); }
    void myClear() { myInComplex.myClear(); myOutCycle.clear(); }
  private:
    FacetComplex myInComplex;
    std::list<facet> myOutCycle;
    auto_ptr<SparsePolyRing> myRingPtr;
  };


  void IsTreeNoOpt::myReadArgs(std::istream& in)
  {
    PolyList PL;
    myRingPtr.reset(new SparsePolyRing(ReadPolyRing(in, GetTag)));
    ReadPolyList(in, PL, *myRingPtr, GetTag);
    myInComplex = FacetComplex(*myRingPtr, PL);
  }

  // ---- CoCoA/TmpIsTree.H  by  M.Caboara ----
  class IsTreeOpt: public ServerOpBase
  {
  public:
    IsTreeOpt(): ServerOpBase(CoCoALib_combinatorics()) {};
    ~IsTreeOpt() {};
    void myOutputSelf(std::ostream& out) const {out << "IsTreeOpt";}
    void myReadArgs(std::istream& in);
    void myCompute() { myOutCycle = myInComplex.myIsTreeOpt(); }
    void myWriteResult(std::ostream& out) const { WritePolyListInVar(out, ourVarName4, FacetList2PolyList(*myRingPtr,myOutCycle)); }
    void myClear() { myInComplex.myClear(); myOutCycle.clear(); }
  private:
    FacetComplex myInComplex;
    std::list<facet> myOutCycle;
    auto_ptr<SparsePolyRing> myRingPtr;
  };


  void IsTreeOpt::myReadArgs(std::istream& in)
  {
    PolyList PL;
    myRingPtr.reset(new SparsePolyRing(ReadPolyRing(in, GetTag)));
    ReadPolyList(in, PL, *myRingPtr, GetTag);
    myInComplex = FacetComplex(*myRingPtr, PL);
  }

  // ---- CoCoA/TmpIsTree.H  by  M.Caboara ----
  class IsTreeCBNoOpt: public ServerOpBase
  {
  public:
    IsTreeCBNoOpt(): ServerOpBase(CoCoALib_combinatorics()) {};
    ~IsTreeCBNoOpt() {};
    void myOutputSelf(std::ostream& out) const {out << "IsTreeCBNoOpt";}
    void myReadArgs(std::istream& in);
    void myCompute() { myOutCycle = myInComplex.myIsTreeCBNoOpt(); }
    void myWriteResult(std::ostream& out) const { WritePolyListInVar(out, ourVarName4, FacetList2PolyList(*myRingPtr,myOutCycle)); }
    void myClear() { myInComplex.myClear(); myOutCycle.clear(); }
  private:
    FacetComplex myInComplex;
    std::list<facet> myOutCycle;
    auto_ptr<SparsePolyRing> myRingPtr;
  };


  void IsTreeCBNoOpt::myReadArgs(std::istream& in)
  {
    PolyList PL;
    myRingPtr.reset(new SparsePolyRing(ReadPolyRing(in, GetTag)));
    ReadPolyList(in, PL, *myRingPtr, GetTag);
    myInComplex = FacetComplex(*myRingPtr, PL);
  }

  // ---- CoCoA/TmpIsTree.H  by  M.Caboara ----
  class IsTreeCBOpt: public ServerOpBase
  {
  public:
    IsTreeCBOpt(): ServerOpBase(CoCoALib_combinatorics()) {};
    ~IsTreeCBOpt() {};
    void myOutputSelf(std::ostream& out) const {out << "IsTreeCBOpt";}
    void myReadArgs(std::istream& in);
    void myCompute() { myOutCycle = myInComplex.myIsTreeCBOpt(); }
    void myWriteResult(std::ostream& out) const { WritePolyListInVar(out, ourVarName4, FacetList2PolyList(*myRingPtr,myOutCycle)); }
    void myClear() { myInComplex.myClear(); myOutCycle.clear(); }
  private:
    FacetComplex myInComplex;
    std::list<facet> myOutCycle;
    auto_ptr<SparsePolyRing> myRingPtr;
  };


  void IsTreeCBOpt::myReadArgs(std::istream& in)
  {
    PolyList PL;
    myRingPtr.reset(new SparsePolyRing(ReadPolyRing(in, GetTag)));
    ReadPolyList(in, PL, *myRingPtr, GetTag);
    myInComplex = FacetComplex(*myRingPtr, PL);
  }

  //----------------------------------------------------
  matrix ApproxPtsToMat(vector<ApproxPts::ApproxPt> Pts)
  {
    size_t NumPts = Pts.size();
    size_t dim = 0;
    if (NumPts != 0) dim = Pts[0].size();
    matrix ResPtsMat = NewDenseMat(RingQ(), NumPts, dim);
    ZZ N, D;
    for (size_t i=0; i < NumPts; ++i)
      for (size_t j=0; j < dim; ++j)
      {
        convert(N, D, Pts[i][j]);
        SetEntry(ResPtsMat, i, j, RingElem(RingQ(),N)/D);
      }
    return ResPtsMat;
  }


 // ---- CoCoA/ApproxBorder.H  by  L.Torrente and J.Abbott ----
  class ApproxBorder: public ServerOpBase
  {
  public:
    ApproxBorder(): ServerOpBase(CoCoALib_approx()) {};
    ~ApproxBorder() {};
    void myOutputSelf(std::ostream& out) const {out << "ApproxBorder";}
    void myReadArgs(std::istream& in);
    void myCompute()  { ApproxPts::SOI(myOutSOI, myOutBBasis, myOutAlmostVanishing, myInPts, myInSemiwidth, myInGamma); }
    void myWriteResult(std::ostream& out) const;
    void myClear() { myOutSOI.clear(); myOutBBasis.clear(); myOutAlmostVanishing.clear(); myInPts.clear(); myInSemiwidth.clear(); }
  private:
    vector<ApproxPts::ApproxPt> myInPts;
    vector<double> myInSemiwidth;
    double myInGamma;
    vector<PPMonoidElem> myOutSOI;
    vector<RingElem> myOutBBasis;
    vector<RingElem> myOutAlmostVanishing;
  };


  void ApproxBorder::myWriteResult(std::ostream& out) const
  {
    out << ourVarName4 << " := Record[" << endl;
    out << " SOI = " << myOutSOI << "," << endl;
    if (!myOutBBasis.empty())
      out << " BBasis = " << myOutBBasis << "," << endl;
    out << " AlmostVanishing = " << myOutAlmostVanishing << endl;
    out << endl << "];" << endl;
  }




  void ApproxBorder::myReadArgs(std::istream& in)
  {
    /*const SparsePolyRing NoUse = */ ReadPolyRing(in, GetTag);
    matrix  PtsMat = ReadRationalMatrix(in, GetTag);
    matrix  SWMat  = ReadRationalMatrix(in, GetTag);
    matrix gamma = ReadRationalMatrix(in, GetTag);
    if (NumRows(SWMat) != 1) CoCoA_ERROR("NumRows should be 1","ApproxBorder");
    if (NumCols(SWMat) != NumCols(PtsMat))
      CoCoA_ERROR("PtsMat and SWMat should have same NumCols","ApproxBorder");
    if (NumRows(gamma) != 1 || NumCols(gamma) != 1)
      CoCoA_ERROR("gamma should be 1x1 matrix","ApproxBorder");
    ZZ N, D;
    vector<ApproxPts::ApproxPt> OrigPts(NumRows(PtsMat), ApproxPts::ApproxPt(NumCols(PtsMat)));
    for (size_t i=0; i < NumRows(PtsMat); ++i)
      for (size_t j=0; j < NumCols(PtsMat); ++j)
      {
        IsRational(N, D, PtsMat(i,j));
        convert(OrigPts[i][j], N,D); // BUG ignores return value.
      }
    vector<double> semiwidth(NumCols(PtsMat));
    for (size_t j=0; j < NumCols(PtsMat); ++j)
    {
      IsRational(N, D, SWMat(0,j));
      convert(semiwidth[j], N,D); // BUG ignores return value.
    }
    IsRational(N,D, gamma(0,0));
    convert(myInGamma, N,D); // BUG ignores return value.
    swap(myInPts, OrigPts);
    swap(myInSemiwidth, semiwidth);
  }



  // ---- CoCoA/ApproxPts.H  by  L.Torrente and J.Abbott ----
  class PreprocessAggr: public ServerOpBase
  {
  public:
    PreprocessAggr(): ServerOpBase(CoCoALib_approx()) {};
    ~PreprocessAggr() {};
    void myOutputSelf(std::ostream& out) const {out << "PreprocessAggr";}
    void myReadArgs(std::istream& in);
    void myCompute()  { ApproxPts::PreprocessAggrAlgm(myOutPts, myInPts, myInSemiwidth); }
    void myWriteResult(std::ostream& out) const { WriteMatrixInVar(out, ourVarName4, ApproxPtsToMat(myOutPts)); }
    void myClear() { myInPts.clear(); myInSemiwidth.clear(); myOutPts.clear(); }
  private:
    vector<ApproxPts::ApproxPt> myInPts, myOutPts;
    vector<double> myInSemiwidth;
  };


  void PreprocessAggr::myReadArgs(std::istream& in)
  {
    /*const SparsePolyRing NoUse = */ ReadPolyRing(in, GetTag);
    matrix  PtsMat = ReadRationalMatrix(in, GetTag);
    matrix  SWMat  = ReadRationalMatrix(in, GetTag);
    if (NumRows(SWMat) != 1) CoCoA_ERROR("NumRows should be 1","PreprocessAggr");
    if (NumCols(SWMat) != NumCols(PtsMat))
      CoCoA_ERROR("PtsMat and SWMat should have same NumCols","PreprocessAggr");
    ZZ N, D;
    vector<ApproxPts::ApproxPt> OrigPts(NumRows(PtsMat), ApproxPts::ApproxPt(NumCols(PtsMat)));
    vector<double> semiwidth(NumCols(PtsMat));
    for (size_t i=0; i < NumRows(PtsMat); ++i)
      for (size_t j=0; j < NumCols(PtsMat); ++j)
      {
        IsRational(N, D, PtsMat(i,j));
        convert(OrigPts[i][j], N, D);
      }
    for (size_t j=0; j < NumCols(PtsMat); ++j)
    {
      IsRational(N, D, SWMat(0,j));
      convert(semiwidth[j], N, D);
    }
    swap(myInPts, OrigPts);
    swap(myInSemiwidth, semiwidth);
  }

  // ---- CoCoA/ApproxPts.H  by  L.Torrente and J.Abbott ----
  class PreprocessSubdiv: public ServerOpBase
  {
  public:
    PreprocessSubdiv(): ServerOpBase(CoCoALib_approx()) {};
    ~PreprocessSubdiv() {};
    void myOutputSelf(std::ostream& out) const {out << "PreprocessSubdiv";}
    void myReadArgs(std::istream& in);
    void myCompute()  { ApproxPts::PreprocessSubdivAlgm(myOutPts, myInPts, myInSemiwidth); }
    void myWriteResult(std::ostream& out) const { WriteMatrixInVar(out, ourVarName4, ApproxPtsToMat(myOutPts)); }
    void myClear() { myInPts.clear(); myInSemiwidth.clear(); myOutPts.clear(); }
  private:
    vector<ApproxPts::ApproxPt> myInPts, myOutPts;
    vector<double> myInSemiwidth;
  };


  void PreprocessSubdiv::myReadArgs(std::istream& in)
  {
    /*const SparsePolyRing NoUse = */ ReadPolyRing(in, GetTag);
    matrix  PtsMat = ReadRationalMatrix(in, GetTag);
    matrix  SWMat  = ReadRationalMatrix(in, GetTag);
    if (NumRows(SWMat) != 1) CoCoA_ERROR("NumRows should be 1","PreprocessSubdiv");
    if (NumCols(SWMat) != NumCols(PtsMat))
      CoCoA_ERROR("PtsMat and SWMat should have same NumCols","PreprocessSubdiv");
    ZZ N, D;
    vector<ApproxPts::ApproxPt> OrigPts(NumRows(PtsMat), ApproxPts::ApproxPt(NumCols(PtsMat)));
    vector<double> semiwidth(NumCols(PtsMat));
    for (size_t i=0; i < NumRows(PtsMat); ++i)
      for (size_t j=0; j < NumCols(PtsMat); ++j)
      {
        IsRational(N, D, PtsMat(i,j));
        convert(OrigPts[i][j], N, D);
      }
    for (size_t j=0; j < NumCols(PtsMat); ++j)
    {
      IsRational(N, D, SWMat(0,j));
      convert(semiwidth[j], N, D);
    }
    swap(myInPts, OrigPts);
    swap(myInSemiwidth, semiwidth);
  }


  // ---- CoCoA/ApproxPts.H  by  L.Torrente and J.Abbott ----
  class PreprocessGrid: public ServerOpBase
  {
  public:
    PreprocessGrid(): ServerOpBase(CoCoALib_approx()) {};
    ~PreprocessGrid() {};
    void myOutputSelf(std::ostream& out) const {out << "PreprocessGrid";}
    void myReadArgs(std::istream& in);
    void myCompute()  { ApproxPts::PreprocessGridAlgm(myOutPts, myInPts, myInSemiwidth); }
    void myWriteResult(std::ostream& out) const { WriteMatrixInVar(out, ourVarName4, ApproxPtsToMat(myOutPts)); }
    void myClear() { myInPts.clear(); myInSemiwidth.clear(); myOutPts.clear(); }
  private:
    vector<ApproxPts::ApproxPt> myInPts, myOutPts;
    vector<double> myInSemiwidth;
  };


  void PreprocessGrid::myReadArgs(std::istream& in)
  {
    /*const SparsePolyRing NoUse = */ ReadPolyRing(in, GetTag);
    matrix  PtsMat = ReadRationalMatrix(in, GetTag);
    matrix  SWMat  = ReadRationalMatrix(in, GetTag);
    if (NumRows(SWMat) != 1) CoCoA_ERROR("NumRows should be 1","PreprocessGrid");
    if (NumCols(SWMat) != NumCols(PtsMat))
      CoCoA_ERROR("PtsMat and SWMat should have same NumCols","PreprocessGrid");
    ZZ N, D;
    vector<ApproxPts::ApproxPt> OrigPts(NumRows(PtsMat), ApproxPts::ApproxPt(NumCols(PtsMat)));
    vector<double> semiwidth(NumCols(PtsMat));
    for (size_t i=0; i < NumRows(PtsMat); ++i)
      for (size_t j=0; j < NumCols(PtsMat); ++j)
      {
        IsRational(N, D, PtsMat(i,j));
        convert(OrigPts[i][j], N, D);
      }
    for (size_t j=0; j < NumCols(PtsMat); ++j)
    {
      IsRational(N, D, SWMat(0,j));
      convert(semiwidth[j], N, D);
    }
    swap(myInPts, OrigPts);
    swap(myInSemiwidth, semiwidth);
  }


// ---- CoCoA/TmpMayerVietorisTree.H  by  E. Saenz-de-Cabezon ----
  class MVTN1: public ServerOpBase
  {
  public:
    MVTN1(): ServerOpBase(CoCoALib_combinatorics()) {};
    ~MVTN1() {};
    void myOutputSelf(std::ostream& out) const {out << "MVT_N-1";}
    void myReadArgs(std::istream& in);
    void myCompute() { MayerVietorisTreeN1(myResPP, myInPP); }
    void myWriteResult(std::ostream& out) const;
    void myClear() { myInPP.clear(); myResPP.clear(); }
  private:
    vector<PPWithMask> myInPP, myResPP;
  };


  void MVTN1::myReadArgs(std::istream& in)
  {
    const SparsePolyRing P(ReadPolyRing(in, GetTag));
    PPOrdering ord = NewLexOrdering(NumIndets(P));
    PPMonoid PPM = NewPPMonoidEv(SymbolRange("x", 0, NumIndets(ord)-1), ord);
    vector<PPMonoidElem> V;
    ReadPPs(in, V, PPM, GetTag);
    DivMaskRule DMR = NewDivMaskEvenPowers();
    vector<long> exps(NumIndets(PPM));
    
    for(unsigned int i=0 ; i<V.size() ; ++i) 
    {
      exponents(exps, V[i]);
      // anna: should find a way to avoid making copies the pps
      myInPP.push_back(PPWithMask(PPMonoidElem(PPM,exps), DMR));
    }
    GlobalOutput() << "Print " << myInPP.size() << ";";
  }
  

  void MVTN1::myWriteResult(std::ostream& out) const
  {
    out << ourVarName4 << " := [];";
    for (unsigned int i=0; i<myResPP.size(); ++i)
      out<< "Append(" << ourVarName4<< ", "<< PP(myResPP[i]) << ");" <<endl;  
  }

//   namespace
//   {
//     class IdealOperationRegistrationClass
//     {
//     public:
//       IdealOperationRegistrationClass()
//       {
//       }
//     } FakeVariable1;
//   }


  
  namespace CoCoAServerOperationsFromCoCoALib
  {
    bool RegisterOps()
    {
      // ideal
      RegisterOp("F5",                ServerOp(new F5GBasis()));
      RegisterOp("ideal_LT",          ServerOp(new IdealLT()));
      RegisterOp("ideal_colon_ideal", ServerOp(new ColonIdId()));
      RegisterOp("ideal_elim",        ServerOp(new IdealElim()));
      RegisterOp("ideal_groebner",    ServerOp(new IdealGBasis()));
      RegisterOp("ideal_sat_groebner",    ServerOp(new IdealSATGBasis()));
      RegisterOp("ideal_satmix_groebner", ServerOp(new IdealSATMixGBasis()));
      RegisterOp("ideal_homogenization",  ServerOp(new IdealHomog()));
      RegisterOp("ideal_intersection",    ServerOp(new IdealIntersection()));
      RegisterOp("ideal_saturation",  ServerOp(new IdealSaturation()));
      RegisterOp("ideal_syzygy",      ServerOp(new IdealSyzygy()));
      // module
      RegisterOp("module_LT",         ServerOp(new ModuleLT()));
      RegisterOp("module_colon_module", ServerOp(new ColonModMod()));
      RegisterOp("module_elim",       ServerOp(new ModuleElim()));
      RegisterOp("module_groebner",   ServerOp(new ModuleGBasis()));
      RegisterOp("module_homogenization", ServerOp(new ModuleHomog()));
      RegisterOp("module_intersection", ServerOp(new ModuleIntersection()));
      RegisterOp("module_saturation", ServerOp(new ModuleSaturation()));
      RegisterOp("module_syzygy",     ServerOp(new ModuleSyzygy()));
      // approx points
      RegisterOp("ApproxBorder",      ServerOp(new ApproxBorder()));
      RegisterOp("PreprocessAggr",    ServerOp(new PreprocessAggr()));
      RegisterOp("PreprocessGrid",    ServerOp(new PreprocessGrid()));
      RegisterOp("PreprocessSubdiv",  ServerOp(new PreprocessSubdiv()));
      // IsTree
      RegisterOp("IsTree_NoOpt",      ServerOp(new IsTreeNoOpt()));
      RegisterOp("IsTree_Opt",        ServerOp(new IsTreeOpt()));
      RegisterOp("IsTree_CBOpt",      ServerOp(new IsTreeCBOpt()));
      RegisterOp("IsTree_CBNoOpt",    ServerOp(new IsTreeCBNoOpt()));
      // Mayer Vietoris Trees
      RegisterOp("MVT_N_minus_one",   ServerOp(new MVTN1()));

      return true;
    }


    bool RegisterOpsOnce()
    {
      static bool EvalOnce = RegisterOps();
      return EvalOnce;
    }
  }
  

}

// $Header: /Volumes/Home/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/RegisterServerOps.C,v 1.11 2008/06/04 18:27:37 abbott Exp $
// $Log: RegisterServerOps.C,v $
// Revision 1.11  2008/06/04 18:27:37  abbott
// Modified the server interface for "SOI": it now accepts a 3rd arg (gamma).
//
// Revision 1.10  2008/05/30 14:20:43  abbott
// SOI now returns also the "almost vanishing" polynomials.
//
// Revision 1.9  2008/05/29 15:46:29  bigatti
// -- added Approximate Border Basis (by Abbott,Torrente)
//
// Revision 1.8  2008/05/28 16:21:05  bigatti
// -- using the new function ReadPPs fom MVTN1
//
// Revision 1.7  2008/05/27 16:22:04  bigatti
// -- added MayerVietorisTreeN1
//
// Revision 1.6  2008/04/18 15:35:57  abbott
// (long overdue) Major revision to matrices
//
// Revision 1.5  2007/11/09 10:45:52  bigatti
// -- [caboara] preparation for self-saturating algorithm
//
// Revision 1.4  2007/10/30 17:14:07  abbott
// Changed licence from GPL-2 only to GPL-3 or later.
// New version for such an important change.
//
// Revision 1.3  2007/09/25 16:32:30  abbott
// Several minor changes to silence gcc-4.3:
//    more #includes,
//    and fixed a template problemm in RegisterServerOps.C
//
// Revision 1.2  2007/09/25 16:28:31  bigatti
// -- ServerOp includes infos about the library it is defined in
// -- CoCoAServer no longer prints its own version
// -- CoCoAServer prints all offered operations
// -- CoCoAServer can change stat_level (Max's verbosity) at runtime
//
// Revision 1.1  2007/04/27 14:54:22  bigatti
// -- content of CoCoAServer.C split into dedicated files
// -- new registration mechanism (through include "RegisterServerOps.H")
//
