//   Copyright (c)  2008  Anna Bigatti and Eduardo Saenz-de-Cabezon

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.

#include "CoCoA/TmpMayerVietorisTree.H"
#include "CoCoA/PPWithMask.H"
#include "CoCoA/TmpPPVector.H"
#include "CoCoA/tmp.H"

#include <iostream>
#include <list>
using std::list;
//#include <vector>
using std::vector;
/**********************************************************/
namespace CoCoA
{

  struct node {
    unsigned int pos;
    unsigned int dim;
    vector<PPWithMask> the_ideal;
    //varlist used_vars;
  };

  typedef list<node> tree;

  namespace // for functions local to this file/compilation unit.
  {

    void tilde(vector<PPWithMask>& g, vector<PPWithMask>& I)
    {
      vector<PPWithMask> m(1,I.back());
      I.pop_back();
      lcms(g,m,I);
      interreduce(g);
    }
  

    bool HasHomology(const PPWithMask& pp, const vector<PPWithMask>& J)
    {
      PPMonoid M=owner(PP(pp));
      int n=NumIndets(M);
      PPMonoidElem ppi=PP(pp)/product(indets(M));
      if (IsDivisible(PPWithMask(ppi,DivMaskImpl(pp)), J))
        return false;
      else
        for (int i=0; i<n; ++i)
          if (!IsDivisible(PPWithMask(indet(M,i)*ppi,DivMaskImpl(pp)), J))
            return false;
      return true;
    }

  }  // end of anonymous namespace


  void MayerVietorisTreeN1(vector<PPWithMask>& betti, const vector<PPWithMask>& I)
  {
    vector<PPWithMask> N1;
    tree T;
    node nodo;
    unsigned int N=NumIndets(owner(PP(I[0]))); //we "know" that the list is not empty
    vector<PPWithMask> J=I;
    interreduce(J);
    std::cout<<" "<<J.size()<<std::endl;
    nodo.pos=1; nodo.dim=0; nodo.the_ideal=J;
    T.push_back(nodo);

    if (J.size()==1) return;

    tree undone=T;
    node new_node;
    int mipos, midim;
    vector<PPWithMask> mi_ideal;
    
    while (!undone.empty())
    {
      swap(mi_ideal, undone.front().the_ideal);
      mipos=undone.front().pos;
      midim=undone.front().dim;
      undone.pop_front();
      
      tilde(new_node.the_ideal, mi_ideal);
      
      //left child
      new_node.pos=mipos*2;
      new_node.dim=midim+1;
      
      if (new_node.dim==N-1)
        for (unsigned int i=0; i<new_node.the_ideal.size(); ++i)
          if (HasHomology(new_node.the_ideal[i], J))
            N1.push_back(new_node.the_ideal[i]);
      if (new_node.the_ideal.size()>1 && new_node.the_ideal.size()+new_node.dim-1>=N-1)
        undone.push_back(new_node);
      
      //right child
      new_node.pos = 1+mipos*2;
      new_node.dim = midim;
      swap(new_node.the_ideal, mi_ideal);
      if (new_node.the_ideal.size()>1 && new_node.the_ideal.size()+new_node.dim-1>=N-1)
        undone.push_back(new_node);
    }
    swap(betti, N1);
  }

}  // end of namespace CoCoA

// RCS header/log in the next few lines
// $Header: /Volumes/Home/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/TmpMayerVietorisTree.C,v 1.1 2008/05/27 16:21:26 bigatti Exp $
// $Log: TmpMayerVietorisTree.C,v $
// Revision 1.1  2008/05/27 16:21:26  bigatti
// -- first import
//
